from .funcs import *
#from .funcbase import CfnYamlDumper, CfnYamlLoader
from .funcbase import yaml
from io import StringIO
from re import sub
#from .funcsold import *
#import yaml
#from yaml import load, dump



def dump(data, stream=None, Dumper=None, **kwds):
    temp = StringIO()
    yaml.dump(data, temp, **kwds)
    
    temp = sub('(?<=\{)Fn::\w*(?=:\ )', 
        lambda matched: "'{}'".format(matched.group()), 
        temp.getvalue(), count=0, flags=0)
    
    if stream is not None:
        stream.write(temp)
    
    return temp
    
def load(stream):
    return yaml.load(stream)
#  return yaml.load(stream, CfnYamlLoader)


from .funcbase import FuncBase
import base64




class Ref(FuncBase):
  argnames = 'logicalName'

class Base64(FuncBase):
  argnames = 'base64str'
  
  @property
  def originalstr(self):
    return base64.b64decode(self.base64str).decode('utf8')
    
  @originalstr.setter
  def originalstr(self, value):
    self.base64str = base64.b64encode(value.encode('utf8')).decode()
  
  def __init__(self, value, isoriginal=False):
    if isoriginal:
      self.originalstr = value
    else:
      self.base64str = value
      
  def __repr__(self):
    tmp = self.originalstr if len(self.originalstr) <= 13 else self.originalstr[0:10] + '...'
    return '<Base64 "{}">'.format(tmp)


class Cidr(FuncBase):
  argnames = ['ipblock', 'count', 'cidrbits']
  reprtemplate = '<Cidr {} / {} ({})>'
  

class FindInMap(FuncBase):
  argnames = ['mapName', 'topLevelKey', 'secondLevelKey']
  reprtemplate = '<FindInMap {}[{}][{}]>'
  
class GetAtt(FuncBase):
  argnames = 'path'

  
class GetAZs(FuncBase):
  argnames = 'region'

class ImportValue(FuncBase):
  argnames = 'sharedValueToImport'
  
class Equals(FuncBase):
  argnames = ['value1', 'value2']
  reprtemplate = '<Equals {} == {}>'
    
class Join(FuncBase):
  argnames = ['delimiter', 'values']
  reprtemplate = '<Join "{}".join({})>'



class Select(FuncBase):
  argnames = ['index', 'values']
  reprtemplate = '<Select {1}[{0}]>'

class Split(FuncBase):
  argnames = ['delimiter', 'sourcestr']
  reprtemplate = '<Select "{1}".split("{0}")>'
    
class Sub(FuncBase):
  argnames = ['template', 'values']
  reprtemplate = '<Sub "{}".format({})>'

class Transform(FuncBase):
  argnames = ['name', 'params']
  
  
######## Condition Functions ###############
    
class And(FuncBase):
  @classmethod
  def from_yaml(cls, loader, node):
    def getv(v):
      return v if not isinstance(v, yaml.Node) else loader.construct_object(v)
    return cls(*[getv(v) for v in node.value])
    
  @classmethod
  def to_yaml(cls, dumper, data):
    return dumper.represent_sequence(cls.yaml_tag, [getattr(data, argname) for argname in cls.argnames])
  
  def __init__(self, *values):
    self.values = values
      
  def __repr__(self):
    return '<And {}>'.format(' && '.join([str(v) for v in self.values]))
  

class Equals(FuncBase):
  argnames = ['value1', 'value2']
  reprtemplate = '<Equals {} == {} >'

class If(FuncBase):
  argnames = ['condition', 'value_if_true', 'value_if_false']
  reprtemplate = '<If {} ? {} : {}>'
  
class Not(FuncBase):
  argnames = ['value']
  reprtemplate = '<Not {}>'
  
class Or(FuncBase):
  @classmethod
  def from_yaml(cls, loader, node):
    def getv(v):
      return v if not isinstance(v, yaml.Node) else loader.construct_object(v)
    return cls(*[getv(v) for v in node.value])
    
  @classmethod
  def to_yaml(cls, dumper, data):
    return dumper.represent_sequence(cls.yaml_tag, [getattr(data, argname) for argname in cls.argnames])

  def __init__(self, *values):
    self.values = values
      
  def __repr__(self):
    return '<Or {}>'.format(' || '.join([str(v) for v in self.values]))
    
    

                  


from ruamel.yaml import YAML, Node

yaml = YAML()



@classmethod
def from_yaml(cls, loader, node):
    def getv(v):
        return v if not isinstance(v, Node) else loader.construct_object(v)
    if isinstance(cls.argnames, str):
        return cls(node.value)
    elif isinstance(cls.argnames, list) or isinstance(cls.argnames, tuple):
        return cls(*[getv(v) for v in node.value])
    else:
        raise ValueError('{}.argnames should be a string or an array'.format(cls.__name__))
    
@classmethod
def to_yaml(cls, dumper, data):
    if isinstance(cls.argnames, str):
        return dumper.represent_scalar(cls.yaml_tag, getattr(data, cls.argnames))
    elif isinstance(cls.argnames, list) or isinstance(cls.argnames, tuple):
        return dumper.represent_sequence(cls.yaml_tag, [getattr(data, argname) for argname in cls.argnames])
    else:
        raise ValueError('{}.argnames should be a string or an array'.format(cls.__name__))
    

def __init__(self, *args):
    names = [self.argnames] if isinstance(self.argnames, str) else self.argnames
#    print('INIT', names, args)
    if len(args) is not len(names):
        raise TypeError('__init__(self, {}) takes {} argumenst, but {} were given'.format(
        ', '.join(names), len(names) + 1, len(args) + 1))
    for i in range(len(names)):
        setattr(self, names[i], args[i])
        
def __repr__(self):
    names = [self.argnames] if isinstance(self.argnames, str) else self.argnames
    return self.reprtemplate.format(*[getattr(self, name) for name in names])

class FuncBaseMetaCls(type):

    def __new__(cls, name, bases, kwds):
        if kwds.get('yaml_tag') is None:
            kwds['yaml_tag'] = '!' + name
            
        if kwds.get('from_yaml') is None:
            kwds['from_yaml'] = from_yaml
            
        if kwds.get('to_yaml') is None:
            kwds['to_yaml'] = to_yaml
            
        if kwds.get('__init__') is None:
            kwds['__init__'] = __init__  
            
        if kwds.get('__repr__') is None:
            kwds['__repr__'] = __repr__
            if kwds.get('reprtemplate') is None:
                if isinstance(kwds['argnames'], str):
                    kwds['reprtemplate'] = '<{} {{}}>'.format(name)
                else:
                    temp = ', '.join(['{}={{}}'.format(name) for name in kwds['argnames']])
                    kwds['reprtemplate'] = '<{} {}>'.format(name, temp)
            
#        print('new!!!!!!!!!!', cls, name, bases, kwds)
        return type.__new__(cls, name, bases, kwds)
        
        
    def __init__(cls, name, bases, kwds):
        super(FuncBaseMetaCls, cls).__init__(name, bases, kwds)
#        if 'yaml_tag' in kwds and kwds['yaml_tag'] is not None:
#            CfnYamlLoader.add_constructor(cls.yaml_tag, cls.from_yaml)
#            CfnYamlDumper.add_representer(cls, cls.to_yaml)
        yaml.register_class(cls)

class FuncBase(metaclass=FuncBaseMetaCls):
    argnames = []
    
    
    
def construct_mapping(self, node, deep=False):
    
    mapping = ODict()

    for key_node, value_node in node.value:
        key = self.construct_object(key_node, deep=deep)
        value = self.construct_object(value_node, deep=deep)

        mapping[key] = value

    return mapping


from setuptools import setup, find_packages

setup(
    name = 'cfn-yaml',
    version = '0.1.1',
    keywords = ('CloudFormation', 'YAML'),
    description = 'Let you load cloudformation template.yaml',
    url = 'https://github.com/Just4test/cfn-yaml',
    license = 'wtfpl',
    install_requires = ['pyyaml'],

    author = 'Just4test',
    author_email = 'myservice@just4test.net',
    
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
    
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3 :: Only'
    ],
    
    packages = find_packages(),
    platforms = 'any'
)